
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripMap()
{
	this.obj = {}
	this.objId = {}
	this.length = 0;
	this.version = yarip.getCurrentVersion();

	this.add = function(page)
	{
		if (!page) return;
		var oldPage = this.obj[page.getName()];
		if (oldPage) oldPage.merge(page);
		else
		{
			this.length++;
			this.obj[page.getName()] = page;
			this.objId[page.getId()] = page;
		}
	}

	this.removeByName = function(name)
	{
		if (!name || !this.obj[name]) return;
		delete this.objId[this.obj[name].getId()];
		delete this.obj[name];
		this.length--;
	}

	this.removeById = function(id)
	{
		if (!id || !this.objId[id]) return;
		delete this.obj[this.objId[id].getName()];
		delete this.objId[id];
		this.length--;
	}

	this.clone = function()
	{
		var map = new this.constructor();
		for (var k in this.obj) if (this.obj[k]) map.add(this.obj[k].clone());
		return map;
	}

	this.replaceExtensionIds = function(map, newId, oldId)
	{
//		if (!map || !newId || !oldId) return;
		for (var p in map.obj)
		{
			var page = map.obj[p];
			for (var k in page.extension.obj)
			{
				if (page.extension.obj[k].getValue() == oldId)
				{
					page.extension.remove(oldId);
					page.extension.add(new YaripItem(newId));
				}
			}
		}
	}

	this.merge = function(map)
	{
		if (!map) return;
		for (var p in map.obj)
		{
			var page = map.obj[p];
			if (this.obj[p]) this.replaceExtensionIds(map, this.obj[p].getId(), page.getId());
			else if (this.objId[page.getId()])
			{
				var newId = "" + yarip.getId();
				this.replaceExtensionIds(map, newId, "" + page.getId());
				page.setId(newId);
			}
		}
//		for (var p in map.obj) this.add(map.obj[p].clone()); // when cloning -> don't change the id!
		for (var p in map.obj) this.add(map.obj[p]);
	}

	this.sort = function(reduce, purge)
	{
		var a = [];
		for (var k in this.obj)
		{
			if (!this.obj[k]) continue;
			this.obj[k].elementWhitelist.sort(purge);
			if (reduce) this.obj[k].elementWhitelist.reduce();
			this.obj[k].elementBlacklist.sort(purge);
			if (reduce) this.obj[k].elementBlacklist.reduce();
			this.obj[k].contentWhitelist.sort(purge);
			this.obj[k].contentBlacklist.sort(purge);
			this.obj[k].styles.sort(purge);
			this.obj[k].extension.sort();
			a.push(this.obj[k]);
		}
		a.sort(this.sortByPage);
		this.obj = {}
		for (var i = 0; i < a.length; i++) this.obj[a[i].getName()] = a[i];
	}

	this.generateXml = function(exporting)
	{
		var r = "<?xml version=\"1.0\" encoding=\"" + yarip.CHARSET + "\"?>\n";
		if (!exporting) r += "\n<!--\nDo not edit this file.\n\nIf you make changes to this file while the application is running,\nthe changes will be overwritten when the application exits.\n-->\n\n";
		r += "<yarip version=\"" + this.version + "\">\n";
		for (var k in this.obj) if (this.obj[k]) r += this.obj[k].generateXml();
		r += "</yarip>";
		return r;
	}

	this.generateCSS = function()
	{
		var r = "\n/* Do not edit this file.\n *\n * If you make changes to this file while the application is running,\n * the changes will be overwritten when the application exits.\n */\n\n";
		for (var k in this.obj) if (this.obj[k]) r += this.obj[k].generateCSS();
		return r;
	}

	this.sortByPage = function(a, b)
	{
		var findDomainRegExp = /((\w+:\/+)?(([\w-.~!$&'()*+,;=:]|%[a-f\d]{2})*@)?)(\b((((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5]))|(\[((([a-f\d]{1,4}:){6}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(::([a-f\d]{1,4}:){5}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(([a-f\d]{1,4})?::([a-f\d]{1,4}:){4}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|((([a-f\d]{1,4}:)?[a-f\d]{1,4})?::([a-f\d]{1,4}:){3}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,2})?[a-f\d]{1,4})?::([a-f\d]{1,4}:){2}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,3})?[a-f\d]{1,4})?::([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,4})?[a-f\d]{1,4})?::([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,5})?[a-f\d]{1,4})?::[a-f\d]{1,4})|(((([a-f\d]{1,4}:){1,6})?[a-f\d]{1,4})?::))\]))|([a-z\d]([\w-.~!$&'()*+,;=]|%[a-f\d]{2})+\.(ac|ad|aero|ae|af|ag|ai|al|am|an|ao|aq|arpa|ar|asia|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|biz|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|cat|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|coop|com|co|cr|cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|edu|ee|eg|er|es|et|eu|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gov|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|info|int|in|io|iq|ir|is|it|je|jm|jobs|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mg|mh|mil|mk|ml|mm|mn|mobi|mo|mp|mq|mr|ms|mt|museum|mu|mv|mw|mx|my|mz|name|na|nc|net|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|org|pa|pe|pf|pg|ph|pk|pl|pm|pn|pro|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|st|su|sv|sy|sz|tc|td|tel|tf|tg|th|tj|tk|tl|tm|tn|to|tp|travel|tr|tt|tv|tw|tz|ua|ug|uk|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|xn--0zwm56d|xn--11b5bs3a9aj6g|xn--80akhbyknj4f|xn--9t4b11yi5a|xn--deba0ad|xn--g6w251d|xn--hgbk6aj7f53bba|xn--hlcj6aya9esc7a|xn--jxalpdlp|xn--kgbechtv|xn--zckzah|ye|yt|yu|za|zm|zw))|([a-z\d]([\w-~!$&'()*+,;=]|%[a-f\d]{2})+))\b(:\d+)?)/i;

		var aArr = a.getName().match(findDomainRegExp)[5].split('.').reverse();
		var bArr = b.getName().match(findDomainRegExp)[5].split('.').reverse();

		if (aArr.length == 1 && bArr.length > 1) return -1;
		if (bArr.length == 1 && aArr.length > 1) return 1;

		for (var i = 1; i >= 0; i--)
		{
			if (aArr[i] < bArr[i]) return -1;
			if (aArr[i] > bArr[i]) return 1;
		}
		for (var i = 2; i < aArr.length && i < bArr.length; i++)
		{
			if (aArr[i] < bArr[i]) return -1;
			if (aArr[i] > bArr[i]) return 1;
		}

		if (aArr.length < bArr.length) return -1;
		if (aArr.length > bArr.length) return 1;

		var aIsDomain = a.getName().search(/^[^:/?&#]+$/) > -1;
		var bIsDomain = b.getName().search(/^[^:/?&#]+$/) > -1;

		if (aIsDomain && bIsDomain) return 0;
		if (aIsDomain && !bIsDomain) return -1;
		if (!aIsDomain && bIsDomain) return 1;

		if (a.getName() < b.getName()) return -1;
		if (a.getName() > b.getName()) return 1;

		return 0;
	}
}
